# SPDX-License-Identifier: WTFPL
# SPDX-FileCopyrightText: 2023 Anna <cyber@sysrq.in>
# No warranty

"""
Common data for Python generators.
"""

#: Project URL names matching upstream bug tracker.
BUG_TRACKER_LABELS = [
    "bug tracker",
    "bug reports",
    "bugtracker",
    "bugs",
    "issue tracker",
    "issues",
    "tracker",
    "github: issues",
]

#: Project URL names matching upstream changelog.
CHANGELOG_LABELS = [
    "changelog",
    "changes",
    "release notes",
]

#: Project URL names matching upstream documentation.
DOCS_LABELS = [
    "doc",
    "docs",
    "documentation",
    "docs: rtd",
]

#: Project URL names matching upstream repository or homepage.
HOME_REPO_LABELS = [
    "code",
    "source",
    "source code",
    "repo",
    "repository",
    "home",
    "homepage",
    "github",
    "github: repo",
]
